@@ -394,12 +394,17 @@ define([
         // Notebook trust events
         this.events.on('trust_changed.Notebook', function (event, trusted) {
             if (trusted) {
-                tnw.set_message(i18n.msg._("Trusted"));
+                tnw.set_message(i18n.msg._("Trusted"), undefined, function() {
+                  return false;
+                }, {'title':'Javascript enabled for notebook display'});
+                // don't allow 'Trusted' button to be clicked
+                $(tnw.selector).attr('disabled', true)
+                $(tnw.selector).css('cursor', 'help');
             } else {
                 tnw.set_message(i18n.msg._("Not Trusted"), undefined, function() {
                   that.notebook.trust_notebook();
                   return false;
-                });
+                }, {'title':'Javascript disabled for notebook display'});
             }
         });
     };
